note
	description: "Class reflective meta data"

	author: "Finnian Reilly"
	copyright: "Copyright (c) 2001-2017 Finnian Reilly"
	contact: "finnian at eiffel hyphen loop dot com"

	license: "MIT license (See: en.wikipedia.org/wiki/MIT_License)"
	date: "2017-12-28 18:45:36 GMT (Thursday 28th December 2017)"
	revision: "2"

class
	EL_CLASS_META_DATA

inherit
	REFLECTED_REFERENCE_OBJECT
		export
			{NONE} all
		redefine
			make, enclosing_object
		end

	EL_REFLECTOR_CONSTANTS

	EL_MODULE_EIFFEL

	EL_MODULE_NAMING

	EL_STRING_CONSTANTS

create
	make

feature {NONE} -- Initialization

	make (a_enclosing_object: like enclosing_object)
		do
			Precursor (a_enclosing_object)
			create cached_field_indices_set.make_equal (3)
			excluded_fields := new_field_indices_set (a_enclosing_object.Except_fields)
			hidden_fields := new_field_indices_set (a_enclosing_object.Hidden_fields)
			field_array := new_field_list.to_array
			field_table := new_field_table
			exported_names := new_exported_names
		end

feature -- Access

	excluded_fields: EL_FIELD_INDICES_SET

	exported_names: ARRAY [STRING]

	field_array: ARRAY [EL_REFLECTED_FIELD]

	field_name_list: EL_STRING_LIST [STRING]
		local
			i: INTEGER
		do
			create Result.make (field_array.count)
			from i := 1 until i > field_array.count loop
				Result.extend (field_array.item (i).name)
				i := i + 1
			end
		end

	field_table: EL_REFLECTED_FIELD_TABLE

	hidden_fields: EL_FIELD_INDICES_SET

	sink_except (a_object: EL_REFLECTIVE; sinkable: EL_DATA_SINKABLE; field_names: STRING)
		local
			excluded: like new_field_indices_set; fields: like field_array
			i: INTEGER
		do
			excluded := new_field_indices_set (field_names)
			fields := field_array
			from i := 1 until i > fields.count loop
				if not excluded.has (i) then
					fields.item (i).write (a_object, sinkable)
				end
				i := i + 1
			end
		end

feature -- Basic operations

	print_fields (a_object: EL_REFLECTIVE; lio: EL_LOGGABLE)
		local
			line_length, length: INTEGER
			name: STRING; value: ZSTRING; l_field: EL_REFLECTED_FIELD
		do
			value := String_8_pool.new_string
			across field_array as fld loop
				l_field := fld.item
				if not hidden_fields.has (l_field.index) then
					name := l_field.name
					value.wipe_out
					value.append_string_general (l_field.to_string (a_object))
					length := name.count + value.count + 2
					if line_length > 0 then
						lio.put_string (", ")
						if line_length + length > Info_line_length then
							lio.put_new_line
							line_length := 0
						end
					end
					lio.put_labeled_string (name, value)
					line_length := line_length + length
				end
			end
			lio.put_new_line
		end

feature -- Comparison

	all_fields_equal (a_current, other: EL_REFLECTIVE): BOOLEAN
		local
			i, count: INTEGER; array: like field_array
		do
			array := field_array; count := array.count
			Result := True
			from i := 1 until not Result or i > count loop
				Result := array.item (i).are_equal (a_current, other)
				i := i + 1
			end
		end

feature {NONE} -- Factory

	new_exported_names: ARRAY [STRING]
		-- Adapt field name with a different word separator
		local
			table: EL_REFLECTED_FIELD_TABLE; l_name: STRING
			l_export_name: like Naming.default_export
		do
			table := field_table
			l_export_name := enclosing_object.export_name
			create Result.make_filled (Empty_string_8, 1, field_count)
			from table.start until table.after loop
				if Naming.is_default (l_export_name) then
					l_name := table.key_for_iteration
				else
					create l_name.make (table.key_for_iteration.count)
					l_export_name (table.key_for_iteration, l_name)
				end
				Result [table.item_for_iteration.index] := l_name
				table.forth
			end
		end

	new_field_indices_set (field_names: STRING): EL_FIELD_INDICES_SET
		local
			cached: like cached_field_indices_set
		do
			cached := cached_field_indices_set
			cached.search (field_names)
			if cached.found then
				Result := cached.found_item
			else
				create Result.make (Current, field_names)
				cached.extend (Result, field_names)
			end
		end

	new_field_list: ARRAYED_LIST [EL_REFLECTED_FIELD]
		local
			i: INTEGER; name: STRING; excluded: like excluded_fields
		do
			excluded := excluded_fields
			create Result.make (field_count - excluded.count)
			from i := 1 until i > field_count loop
				if not excluded.has (i) and then enclosing_object.field_included (Current, i) then
					name := field_name (i)
					-- if not a once ("OBJECT") field
					if name [1] /= '_' then
						name.prune_all_trailing ('_')
						Result.extend (new_reflected_field (i, name))
					end
				end
				i := i + 1
			end
		end

	new_field_table: EL_REFLECTED_FIELD_TABLE
		local
			i, count: INTEGER; l_item: EL_REFLECTED_FIELD
		do
			count := field_array.count
			create Result.make (count, enclosing_object.import_name)
			from i := 1 until i > count loop
				l_item := field_array [i]
				Result.extend (l_item, l_item.name)
				i := i + 1
			end
		end

	new_reference_field (index: INTEGER; name: STRING): EL_REFLECTED_FIELD
		local
			type_table: like Reference_type_table
		do
			type_table := Reference_type_table
			type_table.search (reference_type_id (index))
			if type_table.found then
				Result := new_reflected_field_for_type (type_table.found_item, index, name)
			else
				create {EL_REFLECTED_REFERENCE} Result.make (enclosing_object, index, name)
			end
		end

	new_reflected_field (index: INTEGER; name: STRING): EL_REFLECTED_FIELD
		local
			type: INTEGER
		do
			type := field_type (index)
			if type = Reference_type then
				Result := new_reference_field (index, name)
			else
				Result := new_reflected_field_for_type (Expanded_field_types [type], index, name)
			end
		end

	new_reflected_field_for_type (type: TYPE [EL_REFLECTED_FIELD]; index: INTEGER; name: STRING): EL_REFLECTED_FIELD
		do
			if attached {EL_REFLECTED_FIELD} Eiffel.new_instance_of (type.type_id) as new_field then
				Result := new_field
				Result.make (enclosing_object, index, name)
			end
		end

feature {NONE} -- Implementation

	reference_type_id (index: INTEGER): INTEGER
		local
			i: INTEGER; makeable_from_string: BOOLEAN; makeable: like Type_id_make_from_string
		do
			Result := field_static_type (index)
			if not String_types.has (Result) then
				if field_conforms_to (Result, Boolean_ref_type) then
					Result := Boolean_ref_type
				-- Check if field conforms to `EL_MAKEABLE_FROM_STRING'
				elseif field_conforms_to (Result, Makeable_from_string_type) then
					makeable := Type_id_make_from_string
					from i := 1 until makeable_from_string or i > makeable.count loop
						if field_conforms_to (Result, makeable.integer_32_item (i)) then
							makeable_from_string := True
							Result := makeable.integer_32_item (i)
						else
							i := i + 1
						end
					end
					check makeable: makeable_from_string end
				else
					Result := 0
				end
			end
		end

feature {NONE} -- Internal attributes

	cached_field_indices_set: HASH_TABLE [EL_FIELD_INDICES_SET, STRING]

	enclosing_object: EL_REFLECTIVE

feature {NONE} -- Constants

	Info_line_length: INTEGER
		once
			Result := 100
		end

	Reference_type_table: EL_HASH_TABLE [TYPE [EL_REFLECTED_REFERENCE], INTEGER]
		once
			create Result.make (<<
				[Boolean_ref_type, {EL_REFLECTED_BOOLEAN_REF}],

				[String_z_type, {EL_REFLECTED_ZSTRING}],
				[String_8_type, {EL_REFLECTED_STRING_8}],
				[String_32_type, {EL_REFLECTED_STRING_32}],

				[Type_id_make_from_string.date_time, {EL_REFLECTED_DATE_TIME}],
				[Type_id_make_from_string.zstring, {EL_REFLECTED_MAKEABLE_FROM_ZSTRING}],
				[Type_id_make_from_string.string_8, {EL_REFLECTED_MAKEABLE_FROM_STRING_8}],
				[Type_id_make_from_string.string_32, {EL_REFLECTED_MAKEABLE_FROM_STRING_32}]
			>>)
		end

	frozen Expanded_field_types: ARRAY [TYPE [EL_REFLECTED_FIELD]]
		once
			create Result.make_filled ({EL_REFLECTED_REFERENCE}, 0, 16)
				-- Characters
				Result [Character_8_type] := {EL_REFLECTED_CHARACTER_8}
				Result [Character_32_type] := {EL_REFLECTED_CHARACTER_32}

				-- Integers
				Result [Integer_8_type] := {EL_REFLECTED_INTEGER_8}
				Result [Integer_16_type] := {EL_REFLECTED_INTEGER_16}
				Result [Integer_32_type] := {EL_REFLECTED_INTEGER_32}
				Result [Integer_64_type] := {EL_REFLECTED_INTEGER_64}

				-- Naturals
				Result [Natural_8_type] := {EL_REFLECTED_NATURAL_8}
				Result [Natural_16_type] := {EL_REFLECTED_NATURAL_16}
				Result [Natural_32_type] := {EL_REFLECTED_NATURAL_32}
				Result [Natural_64_type] := {EL_REFLECTED_NATURAL_64}

				-- Reals
				Result [Real_32_type] := {EL_REFLECTED_REAL_32}
				Result [Real_64_type] := {EL_REFLECTED_REAL_64}

				-- Others
				Result [Boolean_type] := {EL_REFLECTED_BOOLEAN}
				Result [Pointer_type] := {EL_REFLECTED_POINTER}
			end

	frozen Type_id_make_from_string: TUPLE [date_time, zstring, string_8, string_32: INTEGER]
		once
			create Result
			Result.date_time := ({EL_DATE_TIME}).type_id
			Result.zstring := ({EL_MAKEABLE_FROM_ZSTRING}).type_id
			Result.string_8 := ({EL_MAKEABLE_FROM_STRING_8}).type_id
			Result.string_32 := ({EL_MAKEABLE_FROM_STRING_32}).type_id
		ensure
			date_time_first: Result.integer_32_item (1) = Result.date_time
		end

end
