note
	description: "Summary description for {EL_REFLECTED_FIELD}."

	author: "Finnian Reilly"
	copyright: "Copyright (c) 2001-2017 Finnian Reilly"
	contact: "finnian at eiffel hyphen loop dot com"

	license: "MIT license (See: en.wikipedia.org/wiki/MIT_License)"
	date: "2017-12-28 12:32:42 GMT (Thursday 28th December 2017)"
	revision: "4"

deferred class
	EL_REFLECTED_FIELD

inherit
	REFLECTED_REFERENCE_OBJECT
		rename
			make as make_reflected,
			is_expanded as is_current_expanded,
			class_name as object_class_name
		export
			{NONE} all
		end

	EL_REFLECTOR_CONSTANTS

	EL_MODULE_EIFFEL

	EL_STRING_CONSTANTS

feature {EL_CLASS_META_DATA} -- Initialization

	make (a_object: EL_REFLECTIVE; a_index: INTEGER; a_name: STRING)
		do
			make_reflected (a_object)
			index := a_index; name := a_name
			type := field_type (index)
			type_id := field_static_type (index)
		end

feature -- Access

	index: INTEGER

	name: STRING

	class_name (a_object: EL_REFLECTIVE): STRING
		do
			Result := Eiffel.class_name_of_type (type_id)
		end

	to_string (a_object: EL_REFLECTIVE): READABLE_STRING_GENERAL
		deferred
		end

	type: INTEGER
		-- abstract type

	type_id: INTEGER
		-- generating type

	value (a_object: EL_REFLECTIVE): like field_value
		do
			enclosing_object := a_object
			Result := field_value (index)
		end

feature -- Status query

	is_expanded: BOOLEAN
		deferred
		end

feature -- Comparison

	are_equal (a_current, other: EL_REFLECTIVE): BOOLEAN
		deferred
		end

feature -- Basic operations

	print_meta_data (a_object: EL_REFLECTIVE; lio: EL_LOGGABLE; i: INTEGER; last: BOOLEAN)
		local
			label: STRING
		do
			label := new_numbered_label (i); label.append (name)
			lio.put_labeled_string (label, class_name (a_object))
			if not last then
				lio.put_new_line
			end
		end

	set (a_object: EL_REFLECTIVE; a_value: like field_value)
		deferred
		end

	set_default_value (a_object: EL_REFLECTIVE)
		do
		end

	set_from_readable (a_object: EL_REFLECTIVE; readable: EL_READABLE)
		deferred
		end

	set_from_integer (a_object: EL_REFLECTIVE; a_value: INTEGER)
		deferred
		end

	set_from_string (a_object: EL_REFLECTIVE; string: READABLE_STRING_GENERAL)
		deferred
		end

	write (a_object: EL_REFLECTIVE; writeable: EL_WRITEABLE)
		deferred
		end

feature -- Element change

	set_index (a_index: like index)
		do
			index := a_index
		end

feature {NONE} -- Implementation

	field_value (a_index: INTEGER): ANY
		deferred
		end

	new_numbered_label (i: INTEGER): STRING
		do
			Result := Once_label
			Result.wipe_out
			if i < 10 then
				Result.append_character (' ')
			end
			Result.append_integer (i); Result.append (once ". ")
		end

feature {NONE} -- Constants

	Once_label: STRING
		once
			create Result.make_empty
		end
end
