note
	description: "Reflected reference field"

	author: "Finnian Reilly"
	copyright: "Copyright (c) 2001-2017 Finnian Reilly"
	contact: "finnian at eiffel hyphen loop dot com"

	license: "MIT license (See: en.wikipedia.org/wiki/MIT_License)"
	date: "2017-12-28 18:22:53 GMT (Thursday 28th December 2017)"
	revision: "4"

class
	EL_REFLECTED_REFERENCE

inherit
	EL_REFLECTED_FIELD
		redefine
			make, set_default_value
		end

	EL_MODULE_EIFFEL

	EL_SHARED_REFLECTION_MANAGER

create
	make

feature {NONE} -- Initialization

	make (a_object: EL_REFLECTIVE; a_index: INTEGER_32; a_name: STRING_8)
		local
			table: like Reflection_manager.default_values_by_type_id
		do
			Precursor (a_object, a_index, a_name)
			if default_defined then
				set_default
			else
				table := Reflection_manager.default_values_by_type_id
				table.search (field_static_type (index))
				if table.found then
					initialize_default (table.found_item)
				else
					initialize_default (Default_any)
				end
			end
		end

feature -- Access

	to_string (a_object: EL_REFLECTIVE): READABLE_STRING_GENERAL
		do
			Result := value (a_object).out
		end

feature -- Status query

	Is_expanded: BOOLEAN = False

feature -- Element change

	set (a_object: EL_REFLECTIVE; a_value: ANY)
		do
			enclosing_object := a_object
			set_reference_field (index, a_value)
		end

	set_from_readable (a_object: EL_REFLECTIVE; a_value: EL_READABLE)
		do
		end

	set_from_string (a_object: EL_REFLECTIVE; string: READABLE_STRING_GENERAL)
		do
		end

	write (a_object: EL_REFLECTIVE; writable: EL_WRITEABLE)
		do
		end

feature -- Comparison

	are_equal (a_current, other: EL_REFLECTIVE): BOOLEAN
		do
			Result := value (a_current).is_equal (value (other))
		end

feature {NONE} -- Implementation

	default_defined: BOOLEAN
		do
		end

	field_value (a_index: INTEGER): ANY
		do
			Result := reference_field (a_index)
		end

	initialize_default (a_default: ANY)
		do
			if attached {like default_value} a_default as like_default then
				default_value := like_default
			end
		end

	initialize_reflection
		do
		end

	set_default
		do
		end

	set_default_value (a_object: EL_REFLECTIVE)
		do
			if default_value /= Default_any then
				enclosing_object := a_object
				set_reference_field (index, default_value)
			end
		end

	set_from_integer (a_object: EL_REFLECTIVE; a_value: INTEGER_32)
			-- Internal attributes
		do
			set (a_object, a_value.to_reference)
		end

feature {NONE} -- Internal attributes

	default_value: ANY

feature {NONE}

	Default_any: ANY
		once
			create Result
		end

end

