note
	description: "Summary description for {EL_REFLECTED_TUPLE}."

	author: "Finnian Reilly"
	copyright: "Copyright (c) 2001-2017 Finnian Reilly"
	contact: "finnian at eiffel hyphen loop dot com"

	license: "MIT license (See: en.wikipedia.org/wiki/MIT_License)"
	date: "2017-12-23 9:20:13 GMT (Saturday 23rd December 2017)"
	revision: "2"

class
	EL_REFLECTED_TUPLE

inherit
	EL_REFLECTED_READABLE
		redefine
			field_value, class_name, write, default_defined, set_default, default_value, set_default_value
		end

create
	make

feature -- Access

	class_name (a_object: EL_REFLECTIVE): STRING
		local
			tuple: TUPLE; i: INTEGER
		do
			Result := Precursor (a_object)
			tuple := value (a_object)
			Result.append (" [")
			from i := 1 until i > tuple.count loop
				if i > 1 then
					Result.append (", ")
				end
				Result.append (tuple.item (i).generator)
				i := i + 1
			end
			Result.append_character (']')
		end

feature -- Basic operations

	read (a_object: EL_REFLECTIVE; reader: EL_MEMORY_READER_WRITER)
		do
			read_tuple (value (a_object), reader)
		end

	write (a_object: EL_REFLECTIVE; writeable: EL_WRITEABLE)
		do
			write_tuple (value (a_object), writeable)
		end

	set_default_value (a_object: EL_REFLECTIVE)
		do
			set (a_object, default_value.twin)
		end

feature {NONE} -- Implementation

	field_value (a_index: INTEGER): TUPLE
		do
			if attached {TUPLE} Precursor (a_index) as tuple then
				Result := tuple
			else
				create Result
			end
		end

	read_tuple (tuple: TUPLE; reader: EL_MEMORY_READER_WRITER)
		local
			i: INTEGER
		do
			from i := 1 until i > tuple.count loop
				inspect tuple.item_code (i)
					when {TUPLE}.Character_8_code then
						tuple.put_character (reader.read_character_8, i)

					when {TUPLE}.Character_32_code then
						tuple.put_character_32 (reader.read_character_32, i)

					when {TUPLE}.Boolean_code then
						tuple.put_boolean (reader.read_boolean, i)

					when {TUPLE}.Integer_8_code then
						tuple.put_integer_8 (reader.read_integer_8, i)

					when {TUPLE}.Integer_16_code then
						tuple.put_integer_16 (reader.read_integer_16, i)

					when {TUPLE}.Integer_32_code then
						tuple.put_integer (reader.read_integer_32, i)

					when {TUPLE}.Integer_64_code then
						tuple.put_integer_64 (reader.read_integer_64, i)

					when {TUPLE}.Natural_8_code then
						tuple.put_natural_8 (reader.read_natural_8, i)

					when {TUPLE}.Natural_16_code then
						tuple.put_natural_16 (reader.read_natural_16, i)

					when {TUPLE}.Natural_32_code then
						tuple.put_natural_32 (reader.read_natural_32, i)

					when {TUPLE}.Natural_64_code then
						tuple.put_natural_64 (reader.read_natural_64, i)

					when {TUPLE}.Real_32_code then
						tuple.put_real_32 (reader.read_real_32, i)

					when {TUPLE}.Real_64_code then
						tuple.put_real_64 (reader.read_real_64, i)

					when {TUPLE}.Reference_code then
						if attached {STRING_GENERAL} tuple.reference_item (i) as str then
							if attached {ZSTRING} str then
								tuple.put_reference (reader.read_string, i)
							elseif attached {STRING} str then
								tuple.put_reference (reader.read_string_8, i)
							elseif attached {STRING_32} str then
								tuple.put_reference (reader.read_string_32, i)
							end
						end
				else
				end
				i := i + 1
			end
		end

	set_default
		local
			i, count: INTEGER_32; member_type, l_type: TYPE [ANY]
		do
			if attached {TUPLE} Eiffel.new_instance_of (type_id) as new_tuple then
				default_value := new_tuple
				l_type := default_value.generating_type
				count := l_type.generic_parameter_count
				from i := 1 until i > count loop
					member_type := l_type.generic_parameter_type (i)
					if not member_type.is_expanded then
						if member_type.type_id = String_z_type then
							default_value.put_reference (Empty_string, i)
						elseif member_type.type_id = String_8_type then
							default_value.put_reference (Empty_string_8, i)
						elseif member_type.type_id = String_32_type then
							default_value.put_reference (Empty_string_32, i)
						end
						default_value.compare_objects
					end
					i := i + 1
				end
			end
		end

	write_tuple (tuple: TUPLE; writeable: EL_WRITEABLE)
		local
			i: INTEGER
		do
			from i := 1 until i > tuple.count loop
				inspect tuple.item_code (i)
					when {TUPLE}.Character_8_code then
						writeable.write_character_8 (tuple.character_8_item (i))

					when {TUPLE}.Character_32_code then
						writeable.write_character_32 (tuple.character_32_item (i))

					when {TUPLE}.Boolean_code then
						writeable.write_boolean (tuple.boolean_item (i))

					when {TUPLE}.Integer_8_code then
						writeable.write_integer_8 (tuple.integer_8_item (i))

					when {TUPLE}.Integer_16_code then
						writeable.write_integer_16 (tuple.integer_16_item (i))

					when {TUPLE}.Integer_32_code then
						writeable.write_integer_32 (tuple.integer_32_item (i))

					when {TUPLE}.Integer_64_code then
						writeable.write_integer_64 (tuple.integer_64_item (i))

					when {TUPLE}.Natural_8_code then
						writeable.write_natural_8 (tuple.natural_8_item (i))

					when {TUPLE}.Natural_16_code then
						writeable.write_natural_16 (tuple.natural_16_item (i))

					when {TUPLE}.Natural_32_code then
						writeable.write_natural_32 (tuple.natural_32_item (i))

					when {TUPLE}.Natural_64_code then
						writeable.write_natural_64 (tuple.natural_64_item (i))

					when {TUPLE}.Real_32_code then
						writeable.write_real_32 (tuple.real_32_item (i))

					when {TUPLE}.Real_64_code then
						writeable.write_real_64 (tuple.real_64_item (i))

					when {TUPLE}.Reference_code then
						if attached {READABLE_STRING_GENERAL} tuple.reference_item (i) as str then
							writeable.write_string_general (str)
						end
				else
				end
				i := i + 1
			end
		end

feature {NONE} -- Internal attributes

	default_value: TUPLE

feature {NONE} -- Constants

	Default_defined: BOOLEAN = True

end
