note
	description: "[
		Used in conjunction with `[$source EL_REFLECTIVELY_SETTABLE]' to reflectively set fields
		from name-value pairs, where value conforms to `READABLE_STRING_GENERAL'
	]"

	author: "Finnian Reilly"
	copyright: "Copyright (c) 2001-2017 Finnian Reilly"
	contact: "finnian at eiffel hyphen loop dot com"

	license: "MIT license (See: en.wikipedia.org/wiki/MIT_License)"
	date: "2017-12-28 10:33:23 GMT (Thursday 28th December 2017)"
	revision: "6"

deferred class
	EL_SETTABLE_FROM_STRING

inherit
	STRING_HANDLER
		undefine
			is_equal
		end

feature {NONE} -- Initialization

	make_from_table (field_values: HASH_TABLE [like new_string, STRING])
		do
			make_default
			across field_values as value loop
				set_field (value.key, value.item)
			end
		end

	make_from_zkey_table (field_values: HASH_TABLE [like new_string, ZSTRING])
		-- make from table with keys of type `like new_string'
		local
			name: STRING
		do
			make_default
			create name.make_empty
			across field_values as value loop
				name.wipe_out
				value.key.append_to_string_8 (name)
				set_field (name, value.item)
			end
		end

	make_default
		deferred
		end

feature -- Access

	field_item (name: READABLE_STRING_GENERAL): like new_string
		local
			table: like field_table
		do
			table := field_table
			table.search (name)
			if table.found then
				Result := field_string (table.found_item)
			else
				Result := new_string
			end
		end

feature -- Element change

	set_field (name: READABLE_STRING_GENERAL; value: like new_string)
		local
			table: like field_table
		do
			table := field_table
			table.search (name)
			if table.found then
				table.found_item.set_from_string (current_reflective, value)
			end
		end

	set_field_from_nvp (nvp: like new_string; delimiter: CHARACTER_32)
		-- Set field from name-value pair `nvp' delimited by `delimiter'. For eg. "var=value" or "var: value"
		require
			has_one_equal_sign: nvp.has (delimiter)
		local
			pair: like Name_value_pair
		do
			pair := Name_value_pair
			pair.set_from_string (nvp, delimiter)
			set_field (pair.name, pair.value)
		end

feature {NONE} -- Implementation

	current_reflective: EL_REFLECTIVE
		deferred
		end

	field_string (a_field: EL_REFLECTED_FIELD): STRING_GENERAL
		deferred
		end

	name_value_pair: EL_NAME_VALUE_PAIR [STRING_GENERAL]
		deferred
		end

	new_string: STRING_GENERAL
		deferred
		end

	field_table: EL_REFLECTED_FIELD_TABLE
		deferred
		end

feature {EL_REFLECTION_HANDLER} -- Implementation

	string_type_id: INTEGER
		-- type_id of string {S}
		deferred
		end

end
